$metadata-color: var(--ct-color-subtle-copy);

$listing-ct-hover: var(--ct-color-list-hover-bg);
$listing-ct-hover-icon: var(--ct-color-list-hover-icon);
$listing-ct-active: var(--ct-color-list-active-bg);
$listing-ct-padding: 0.5rem;
$listing-ct-spacing: 1rem;
$listing-ct-open: #f5f5f5;
$listing-ct-open-width: 3px;
$listing-ct-metadata: $metadata-color;
$listing-ct-warning-color: var(--ct-color-light-red);
$listing-ct-border: var(--pf-global--palette--black-300);
$listing-ct-border-light: var(--ct-global--palette--black-250);
$listing-ct-border-maybe: var(--pf-global--palette--black-300-3);
