@use "ct-card.scss";
@import "@patternfly/patternfly/base/patternfly-variables.scss";
@import "@patternfly/patternfly/components/Table/table.scss";
@import "@patternfly/patternfly/components/Table/table-grid.scss";
@import "@patternfly/patternfly/components/Toolbar/toolbar.scss";

/* Style the list cards as ct-cards */
.pf-c-page__main-section .pf-c-card {
    @extend .ct-card;
}

// PF4 issue https://github.com/patternfly/patternfly-react/issues/4612
// Mimic .pf-c-table__action
#virtual-machines-listing td:last-child {
    --pf-c-table--cell--Width: 1%;
}

// Expand the link to the container, for easier clickability
#virtual-machines-listing th a {
    display: block;
    text-align: left;
}

/* Add  the missing space when the list's toolbar wraps
 * patternfly/patternfly#3348
 */
.pf-c-toolbar__content-section {
    gap: var(--pf-global--spacer--sm) 0;
}
