.vcpu-modal-grid {
  display: grid;
  grid-template-columns: repeat(auto-fit, minmax(200px, 1fr));
  grid-row-gap: 1em;
}

.vcpu-modal-grid input[inputmode=numeric],
.vcpu-modal-grid input[type=number] {
    max-width: 7rem;
}

.vcpu-modal-grid {
    line-height: inherit;
}
