.boot-order-modal-cell {
    display: flex;
}

.boot-order-description {
    flex: 1 0 30%;
    flex-basis: 30%;
    font-weight: var(--pf-global--FontWeight--bold);
}

.boot-order-additional-info {
    flex-basis: 70%;
}

// Patternfly horizontal lists should convert to vertical in small screens https://github.com/patternfly/patternfly/issues/3630
@media (max-width: 640px) {
    .boot-order-additional-info .pf-c-description-list {
        --pf-c-description-list__group--GridTemplateColumns: var(--pf-c-description-list--m-vertical__group--GridTemplateColumns);
    }
}
