// imports 'pf-u-pt-md' class used for Expander in Add Disk dialog
@use "../../../../node_modules/@patternfly/patternfly/utilities/Spacing/spacing.css";

.disk-serial {
    font-family: var(--pf-global--FontFamily--monospace);
}
