.resource-state-text {
  margin-right: 1rem;

  // In definition lists (for Overview),
  // match text size and alignment
  .pf-c-table &,
  .pf-c-description-list & {
    // Match text size
    font-size: inherit;
    // Compensate for padding
    position: relative;
    // Using margin to remove the extra padded space,
    // instead of keeping the space and moving contents w/ just left/top
    margin-left: calc(-1 * var(--pf-c-label--PaddingLeft));
    margin-top: calc(-1 * var(--pf-c-label--PaddingTop));
  }
}

.resource-state-text-cell {
  // Add some spacer if multiple labels exist
  row-gap: var(--pf-global--spacer--sm);
}

.pf-c-label.resource-state--shut-off, .pf-c-label.resource-state--inactive {
  background: transparent;
}
