@import '@patternfly/react-styles/css/components/Consoles/SerialConsole.css';

/* Make the VNC canvas scale down to available width */
.pf-c-console__vnc canvas {
    max-width: 100%;
    height: auto !important;
}

/* Hide send key button - there is not way to do that from the JS
 * https://github.com/patternfly/patternfly-react/issues/3689
 */
#pf-c-console__send-shortcut {
    display: none;
}

/* Add spacing between action buttons */
.pf-c-console__actions-vnc {
    gap: var(--pf-global--spacer--sm);
}
