@use "../../node_modules/@patternfly/patternfly/utilities/BoxShadow/box-shadow.css";

.pf-c-page__main-section.ct-no-bottom-padding {
    --pf-c-page__main-section--PaddingBottom: 0;
}

// <Toolbar> embedded into an <Alert>
.pf-c-toolbar.ct-alert-toolbar {
    --pf-c-toolbar--BackgroundColor: transparent;
    --pf-c-toolbar--PaddingBottom: 0;
}
