@import "../../../lib/_global-variables.scss";

.pf-c-table.vm-device-table {
  /* Set overflow with a limiter on the list, so it scrolls instead of the dialog */
  max-height: min(50vh, 50rem);
  overflow: auto;

  /* The list is used within a table; remove excess padding */
  th {
    --pf-c-table--m-compact-th--PaddingTop: 0;
  }
}
