$icon-font-path: 'patternfly-icons-fake-path/';
$font-path: 'patternfly-fonts-fake-path/';

@import "./patternfly-overrides-variables";
@import "patternfly/dist/sass/patternfly";
@import "./patternfly-4-cockpit.scss";
@import "./patternfly-overrides.scss";
