/*
 * This file is part of Cockpit.
 *
 * Copyright (C) 2018 Red Hat, Inc.
 *
 * Cockpit is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * Cockpit is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
 */

// --- Store actions --------------------------------------------
export const ADD_NOTIFICATION = "ADD_NOTIFICATION";
export const ADD_UI_VM = "ADD_UI_VM";
export const CLEAR_NOTIFICATION = "CLEAR_NOTIFICATION";
export const CLEAR_NOTIFICATIONS = "CLEAR_NOTIFICATIONS";
export const DELETE_UI_VM = "DELETE_UI_VM";
export const DELETE_UNLISTED_VMS = "DELETE_UNLISTED_VMS";
export const SET_LOGGED_IN_USER = "SET_LOGGED_IN_USER";
export const SET_NODE_MAX_MEMORY = "SET_NODE_MAX_MEMORY";
export const UNDEFINE_NETWORK = "UNDEFINE_NETWORK";
export const UNDEFINE_STORAGE_POOL = "UNDEFINE_STORAGE_POOL";
export const UNDEFINE_VM = "UNDEFINE_VM";
export const UPDATE_ADD_INTERFACE = "UPDATE_ADD_INTERFACE";
export const UPDATE_ADD_NETWORK = "UPDATE_ADD_NETWORK";
export const UPDATE_ADD_NODE_DEVICE = "UPDATE_ADD_NODE_DEVICE";
export const UPDATE_ADD_VM = "UPDATE_ADD_VM";
export const UPDATE_ADD_STORAGE_POOL = "UPDATE_ADD_STORAGE_POOL";
export const UPDATE_LIBVIRT_VERSION = "UPDATE_LIBVIRT_VERSION";
export const UPDATE_DOMAIN_SNAPSHOTS = "UPDATE_DOMAIN_SNAPSHOTS";
export const UPDATE_OS_INFO_LIST = "UPDATE_OS_INFO_LIST";
export const UPDATE_STORAGE_VOLUMES = "UPDATE_STORAGE_VOLUMES";
export const UPDATE_UI_VM = "UPDATE_UI_VM";
export const UPDATE_VM = "UPDATE_VM";
