.snap-greyed-out {
    color: var(--pf-global--palette--black-600);
}

.tooltip-circle {
    padding: .5rem
}

// CheckIcon is black by default
.snap-creation-time svg {
    color: var(--pf-global--palette--green-300);
}
